
class Router(object):

    def __init__(self, controller, collection, routes=[]):
        self.controller = controller

        self.collection_url = '/' + collection
        self.resource_url = self.collection_url + '/<id>'
        self.resource_enum_url = self.resource_url.replace('<id>', '<id1>')

        if routes:
            self._routes = [
                r for r in self.get_crud() if r['action'] in routes]
        else:
            self._routes = self.get_crud()

    def get_crud(self):
        return [
            {
                'action': 'create',
                'method': 'POST',
                'url': self.collection_url,
                'callback': self.controller.create
            },
            {
                'action': 'get',
                'method': 'GET',
                'url': self.resource_url,
                'callback': self.controller.get
            },
            {
                'action': 'list',
                'method': 'GET',
                'url': self.collection_url,
                'callback': self.controller.list
            },
            {
                'action': 'update',
                'method': 'PUT',
                'url': self.resource_url,
                'callback': self.controller.update
            },
            {
                'action': 'delete',
                'method': 'DELETE',
                'url': self.resource_url,
                'callback': self.controller.delete
            }
        ]

    @property
    def routes(self):
        return self._routes



import flask

from infosystem.common.subsystem.router import Router
from infosystem.common.subsystem.driver import Driver
from infosystem.common.subsystem.manager import Manager
from infosystem.common.subsystem.controller import Controller


class Subsystem(flask.Blueprint):

    def __init__(
            self, resource=None, router=None, controller=None, manager=None,
            driver=None, individual_name=None, collection_name=None,
            operations=[]):

        individual_name = individual_name or resource.individual()
        collection_name = collection_name or resource.collection()

        super().__init__(collection_name, collection_name)

        driver = driver(resource) \
            if driver else Driver(resource) if resource else None
        manager = manager(driver) if manager else Manager(driver)
        controller = controller(manager, individual_name, collection_name) \
            if controller else Controller(
                manager, individual_name, collection_name)
        router = router(controller, collection_name, routes=operations) \
            if router else Router(
                controller, collection_name, routes=operations)

        self.name = collection_name
        self.router = router
        self.manager = manager
        self.register_routes()

    def register_routes(self):
        for route in self.router.routes:
            self.add_url_rule(
                route['url'], view_func=route['callback'],
                methods=[route['method']])



from infosystem.database import db
from sqlalchemy import UniqueConstraint
from infosystem.common.subsystem import entity


class Application(entity.Entity, db.Model):

    DEFAULT = "default"

    attributes = ['name', 'description']
    attributes += entity.Entity.attributes

    name = db.Column(db.String(30), nullable=False)
    description = db.Column(db.String(1000), nullable=False)

    __table_args__ = (UniqueConstraint('name', name='application_name_uk'),)

    def __init__(self, id, name, description,
                 active=True, created_at=None, created_by=None,
                 updated_at=None, updated_by=None, tag=None):
        super().__init__(id, active, created_at, created_by,
                         updated_at, updated_by, tag)
        self.name = name
        self.description = description



import uuid
from sqlalchemy.orm import exc
from infosystem.common import exception


class Driver(object):

    def __init__(self, resource):
        self.resource = resource

    def removeId(self, entity_aux):
        new_id = uuid.uuid4().hex

        if entity_aux.get('id') is not None:
            new_id = entity_aux.pop('id')

        return new_id

    def instantiate(self, **kwargs):
        try:
            embedded = {}
            for attr in self.resource.embedded():
                if attr not in kwargs:
                    raise Exception()
                embedded.update({attr: kwargs.pop(attr)})

            instance = self.resource(**kwargs)

            for attr in embedded:
                value = embedded[attr]
                var = getattr(self.resource, attr)
                # TODO(samueldmq): is this good enough? should we discover it?
                mapped_attr = {self.resource.individual() + '_id': instance.id}
                if isinstance(value, list):
                    setattr(instance, attr, [var.property.mapper.class_(
                        id=self.removeId(ref), **dict(ref, **mapped_attr))
                        for ref in value])
                else:
                    # TODO(samueldmq): id is inserted here. it is in the
                    # manager for the entities. do it all in the resource
                    # contructor
                    setattr(instance, attr, var.property.mapper.class_(
                        id=uuid.uuid4().hex, **dict(value, **mapped_attr)))
        except Exception as exec:
            # TODO(samueldmq): replace with specific exception
            message = ''.join(exec.args)
            raise exception.BadRequest(message)

        return instance

    def create(self, entity, session):
        session.add(entity)
        session.flush()

    def update(self, entity, data, session):
        # try:
        #     entity = self.get(id, session)
        # except exc.NoResultFound:
        #     raise exception.NotFound()

        for attr in self.resource.embedded():
            if attr in data:
                value = data.pop(attr)
                var = getattr(self.resource, attr)
                # TODO(samueldmq): is this good enough? should we discover it?
                mapped_attr = {self.resource.individual() + '_id': id}
                if isinstance(value, list):
                    setattr(entity, attr, [var.property.mapper.class_(
                        id=self.removeId(ref), **dict(ref, **mapped_attr))
                        for ref in value])
                else:
                    # TODO(samueldmq): id is inserted here. it is in the
                    # manager for the entities. do it all in the resource
                    # contructor
                    setattr(entity, attr, var.property.mapper.class_(
                        id=uuid.uuid4().hex, **dict(value, **mapped_attr)))

        for key, value in data.items():
            if hasattr(entity, key):
                try:
                    setattr(entity, key, value)
                except AttributeError:
                    raise exception.BadRequest(
                        f'Error! The attribute {key} is read only')
            else:
                raise exception.BadRequest(
                    f'Error! The attribute {key} not exists')

        session.flush()
        return entity

    def delete(self, entity, session):
        session.delete(entity)
        session.flush()

    def get(self, id, session):
        try:
            query = session.query(self.resource).filter_by(id=id)
            result = query.one()
        except exc.NoResultFound:
            raise exception.NotFound()

        return result

    def list(self, session, **kwargs):
        query = session.query(self.resource)
        for k, v in kwargs.items():
            if hasattr(self.resource, k):
                if isinstance(v, str) and '%' in v:
                    query = query.filter(getattr(self.resource, k).like(v))
                else:
                    query = query.filter(getattr(self.resource, k) == v)

        result = query.all()
        return result

    def count(self, session):
        try:
            rows = session.query(self.resource.id).count()
            result = rows
        except exc.NoResultFound:
            raise exception.NotFound()

        return result



from flask.globals import current_app

from infosystem.celery import celery, decide_on_run
from infosystem.common import exception
from infosystem.subsystem.user.email import TypeEmail


@decide_on_run
@celery.task(autoretry_for=(exception.NotFound,),
             default_retry_delay=5,
             retry_kwargs={'max_retries': 3})
def send_email(user_id: str) -> None:
    return current_app.subsystems['users'].manager.notify(
        id=user_id, type_email=TypeEmail.ACTIVATE_ACCOUNT)



from typing import Dict

from infosystem.common import utils
from infosystem.common.subsystem import Subsystem
from infosystem.subsystem.application.resource import Application


class BootstrapApplication(object):

    def __init__(self, subsystems: Dict[str, Subsystem]):
        self.application_manager = subsystems['applications'].manager

    def execute(self) -> Application:
        application = self._get_application_default()
        return self._save_application(application)

    def _get_application_default(self) -> Application:
        return Application(id=utils.random_uuid(),
                           name=Application.DEFAULT,
                           description="Application Default")

    def _save_application(self, application: Application) -> Application:
        return self.application_manager.create(**application.to_dict())



import os
import shutil

from infosystem.common.subsystem import operation
from infosystem.subsystem.file import manager
from infosystem.subsystem.image import tasks
from infosystem.subsystem.image.resource import QualityImage


class Create(manager.Create):

    def __call__(self, file, **kwargs):
        return super().__call__(file=file, **kwargs)

    def post(self):
        tasks.process_image(self.upload_folder, self.entity.filename)


class Get(operation.Get):

    def pre(self, session, id, **kwargs):
        self.quality = kwargs.pop('quality', QualityImage.min)

        return super().pre(id=id, session=session)

    def do(self, session, **kwargs):
        file = super().do(session=session, **kwargs)

        filename = file.filename_with_quality(self.quality)
        folder = self.manager.get_upload_folder(file, file.domain_id)

        return folder, filename


class Delete(operation.Delete):

    def post(self):
        # TODO(fdoliveira) Put this in worker
        folder = self.manager.get_upload_folder(self.entity,
                                                self.entity.domain_id)
        shutil.rmtree(folder)


class Manager(manager.Manager):
    ALLOWED_EXTENSIONS = ['jpg', 'jpeg', 'png', 'gif']

    def __init__(self, driver):
        super().__init__(driver)
        self.create = Create(self)
        self.get = Get(self)
        self.delete = Delete(self)

    def get_upload_folder(self, entity, domain_id):
        base_folder = self._get_base_folder()
        entity_name = type(entity).__name__
        folder = os.path.join(base_folder,
                              entity_name,
                              entity.type_image,
                              self.OPTIONAL_FOLDER,
                              domain_id,
                              entity.id)
        if not os.path.exists(folder):
            os.makedirs(folder)
        return folder



from setuptools import setup, find_packages

REQUIRED_PACKAGES = [
    'apscheduler',
    'flask',
    'flask-rbac',
    'flask-sqlalchemy',
    'flask-migrate',
    'gabbi',
    'pika',
    'sparkpost',
    'celery',
    'pillow'
]

setup(
    name='infosystem',
    version='1.1.1',
    summary='Infosystem Framework',
    url='https://github.com/objetorelacional/infosystem',
    author='Samuel de Medeiros Queiroz, Francois Oliveira',
    author_email='samueldmq@gmail.com, oliveira.francois@gmail.com',
    license='Apache-2',
    packages=find_packages(exclude=["tests"]),
    install_requires=REQUIRED_PACKAGES
)



from infosystem.common.subsystem import router


class Router(router.Router):

    def __init__(self, controller, collection, routes=[]):
        super().__init__(controller, collection, routes)

    @property
    def routes(self):
        return [
            {
                'action': 'create',
                'method': 'POST',
                'url': self.collection_url,
                'callback': self.controller.create
            },
            {
                'action': 'get',
                'method': 'GET',
                'bypass': True,
                'url': self.resource_url,
                'callback': self.controller.get
            },
            {
                'action': 'list',
                'method': 'GET',
                'url': self.collection_url,
                'callback': self.controller.list
            },
            {
                'action': 'update',
                'method': 'PUT',
                'url': self.resource_url,
                'callback': self.controller.update
            },
            {
                'action': 'delete',
                'method': 'DELETE',
                'url': self.resource_url,
                'callback': self.controller.delete
            }
        ]
